@@ -1,6 +1,6 @@
 import os, re, math
 import numpy as np
-from shapely.geometry import Polygon, LineString, box
+from shapely.geometry import Polygon, LineString, box, LinearRing
 from lxml import etree as ET
 from skimage import transform as tf
 from collections import OrderedDict
@@ -75,6 +75,14 @@ class Contour:
         '''Allows use of != between multiple contours.'''
         return self.output() != other.output()
 # Helper Functions
+    def isReverse(self):
+        '''Returns true if contour is a reverse trace (negative area)'''
+        self.popshape()
+        if self.closed:
+            ring = LinearRing(self._shape.exterior.coords) # convert polygon to ring
+            return not ring.is_ccw # For some reason, the opposite is true (image vs biological coordinate system?)
+        else:
+            return False
     def overlaps(self, other, threshold=(1+2**(-17))):
         '''Return 0 if no overlap.
         For closed traces: return 1 if AoU/AoI < threshold, return AoU/AoI if not < threshold
@@ -238,7 +246,7 @@ class Contour:
             tupList.append(tup)
         return tupList
 # Accessors
-    def getLength(self): #===
+    def getLength(self):
         '''Returns the sum of all line segments in the contour object'''
         length = 0
         for index in range( len(self.points) ):
@@ -663,6 +671,18 @@ class Section:
         '''Returns attributes in xml output format'''
         return str(self.index), str(self.thickness), str(self.alignLocked).lower()
 # Mutators
+    def renameObject(self, oldName, newName):
+        '''Renames all occurances of oldName in section to newName'''
+        for contour in self.contours:
+            if contour.name == oldName:
+                contour.name = newName
+    def getObject(self, regex):
+        '''Returns a list of objects, in this section, that match the regex'''
+        objects = []
+        for contour in self.contours:
+            if regex.match(contour.name) != None:
+                objects.append(contour)
+        return objects
     def s2b(self, string):
         '''Converts string to bool'''
         if string == 'None':
@@ -712,7 +732,7 @@ class Series:
         self.tag = 'Series'
         
         self.contours = self.popcontours(root)
-        self.sections = [] #Sorted in self.getSectionsXML()
+        self.sections = [] #Sorted by index in self.getSectionsXML()
         
         self.index = self.popindex(root)
         self.viewport = self.popviewport(root)
@@ -838,6 +858,97 @@ class Series:
         return (self.output()[0] != other.output()[0] and
                 self.output()[1] != other.output()[1])
 # Accessors
+    def locateReverseTraces(self):
+        reverseDict = {}
+        for section in self.sections:
+            revTraces = []
+            for contour in section.contours:
+                if contour.isReverse():
+                    revTraces.append(contour)
+            if len(revTraces) != 0:
+                reverseDict[section.index] = revTraces
+        return reverseDict
+    def locateDistantTraces(self, threshold=7):
+        '''Returns a dictionary of indexes containing traces that exist after <threshold (def: 7)> sections of non-existence'''
+        # Build a list of lists for all the contours in each section
+        allSectionContours = []
+        for section in self.sections:
+            contours = list(set([cont.name for cont in section.contours]))
+            allSectionContours.append(contours)
+        # Go through list of contours and check for distances
+        index = int(self.sections[0].index)
+        distantTraces = {}
+        for sec in range(len(allSectionContours)):
+            traces = []
+            for contour in allSectionContours[sec]:
+                # Check above
+                if sec+threshold+1 <= len(self.sections):
+                    # Check and ignore if in section:section+threshold
+                    sectionToThresholdContours = [] 
+                    for contList in allSectionContours[sec+1:sec+threshold+1]:
+                        sectionToThresholdContours.extend(contList)
+                    if contour not in sectionToThresholdContours:
+                        # Check if contour is in section+threshold and up
+                        thresholdToEndContours = []
+                        for contList in allSectionContours[sec+threshold+1:]:
+                            thresholdToEndContours.extend(contList)
+                        if contour in thresholdToEndContours:
+                            print('frome above %s')%contour
+                            traces.append(contour)
+                # Check below
+                if sec-threshold-1 >= 0:
+                    # Check and ignore if in section-threshold:section
+                    minusThresholdToSectionContours = []
+                    for contList in allSectionContours[sec-threshold:sec]:
+                        minusThresholdToSectionContours.extend(contList)
+                    if contour not in minusThresholdToSectionContours:
+                        # Check if contour is in section-threshold and down
+                        beginToMinusThresholdContours = []
+                        for contList in allSectionContours[:sec-threshold]:
+                            beginToMinusThresholdContours.extend(contList)
+                        if contour in beginToMinusThresholdContours:
+                            print('frome below %s')%contour
+                            traces.append(contour)
+                # Add traces to distantTraces dictionary
+                if len(traces) != 0:
+                    distantTraces[index] = traces
+            index += 1
+        return distantTraces
+    def locateDuplicates(self):
+        '''Locates overlapping traces of the same name in a section. Returns a dictionary of section numbers with duplicates'''
+        # Build dictionary of sections and contours that have same name
+        duplicateNames = {}
+        for section in self.sections:
+            duplicates = []
+            contourNames = [cont.name for cont in section.contours] # List of contour names
+            # Go through each contour, see if name appears in contourName > 1 time
+            for contour in section.contours:
+                if contourNames.count(contour.name) > 1:
+                    duplicates.append(contour)
+            if len(duplicates) > 0:
+                duplicateNames[section.index] = duplicates
+        # Go through each section in duplicateNames
+        duplicateDict = {}
+        for section in duplicateNames:
+            duplicates = []
+            for contour in duplicateNames[section]:
+                # Filter contours of same memory address
+                copyContours = [cont for cont in duplicateNames[section] if id(cont) != id(contour)]
+                for cont in copyContours:
+                    if contour.overlaps(cont) != 0:
+                        duplicates.append(cont)
+            duplicateDict[section] = duplicates
+        return duplicateDict
+    def renameObject(self, oldName, newName):
+        '''Renames all occurances of oldName in a series to newName.'''
+        for section in self.sections:
+            section.renameObject(oldName, newName)
+    def getObject(self, regex):
+        '''Returns a list of 1 list per section containing all the contour that match the regex'''
+        objects = []
+        for section in self.sections:
+            section.append(section.getObject(regex))
+        return objects  
     def getData(self, object_name, data_string):
         string = str(data_string).lower()
         if string == 'volume':
@@ -854,7 +965,7 @@ class Series:
             return self.getStartEndCount(object_name)[1]
         elif string == 'count':    
             return self.getStartEndCount(object_name)[2]
-    def getObjectLists(self): #=== added ',' to {2}
+    def getObjectLists(self):
         '''Returns sorted lists of dendrite names, protrusion names, trace names, and a list of other objects in series'''
         dendrite_expression = 'd[0-9]{2,}' # represents base dendrite name (d##)
         protrusion_expression = 'd[0-9]{2,}p[0-9]{2,}$' # represents base protrusion name (d##p##)
@@ -874,7 +985,7 @@ class Series:
             for contour in section.contours:
                 # Dendrite
                 if dendrite_expression.match(contour.name) != None:
-                    dendrites.append(contour.name[0:dendrite_expression.match(contour.name).end()]) #===
+                    dendrites.append(contour.name[0:dendrite_expression.match(contour.name).end()])
                 # Protrusion
                 if protrusion_expression.match(contour.name) != None:
                     protrusions.append(contour.name)
